#include "tcp_functions.h"

#define LISTENQ 1024		/* Maximum clients to queue */

void Close(int sockfd) {
  close (sockfd);
}

int Listen(int sockfd, int backlog) {
  return (listen(sockfd, backlog));
}

char *gai_strerror(int error) {
   return(strdup("Networking Error\n"));
}

int Setsockopt(int listenfd, int level, int optname, const void *opt_val, int optlen) {
   return (setsockopt(listenfd, level, optname, opt_val, optlen)); 
}

int tcp_connect(char *host, char *serv)
{
        int sockfd, n;
        struct addrinfo hints, *res, *ressave;

        bzero(&hints, sizeof(struct addrinfo));
        hints.ai_family = AF_UNSPEC;
        hints.ai_socktype = SOCK_STREAM;

        if ( (n = getaddrinfo(host, serv, &hints, &res)) != 0) {
                fprintf(stderr, "tcp_connect error for %s, %s: %s", host, serv, gai_strerror(n));
		exit (-1);
	}
        ressave = res;

        do {
                sockfd = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
                if (sockfd < 0)
                        continue;       /* ignore this one */

                if (connect(sockfd, res->ai_addr, res->ai_addrlen) == 0)
                        break;          /* success */

                Close(sockfd);  /* ignore this one */
        } while ( (res = res->ai_next) != NULL);

        if (res == NULL) {       /* errno set from final connect() */
                fprintf(stderr, "tcp_connect error for %s, %s", host, serv);
		exit(-1);
	}

        freeaddrinfo(ressave);

        return(sockfd);
}


int
tcp_listen(const char *host, const char *serv, socklen_t *addrlenp)
{
        int listenfd, n;
        const int on = 1;
        struct addrinfo hints, *res, *ressave;

        bzero(&hints, sizeof(struct addrinfo));
        hints.ai_flags = AI_PASSIVE;
        hints.ai_family = AF_UNSPEC;
        hints.ai_socktype = SOCK_STREAM;

        if ( (n = getaddrinfo(host, serv, &hints, &res)) != 0) {
                fprintf(stderr, "tcp_listen error for %s, %s: %s", host, serv, gai_strerror(n));
		exit (-1);
	}
        ressave = res;

        do {
                listenfd = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
                if (listenfd < 0)
                        continue;               /* error, try next one */

                Setsockopt(listenfd, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on));
                if (bind(listenfd, res->ai_addr, res->ai_addrlen) == 0)
                        break;                  /* success */

                Close(listenfd);        /* bind error, close and try next one */
        } while ( (res = res->ai_next) != NULL);

        if (res == NULL) {      /* errno from final socket() or bind() */
                fprintf(stderr, "tcp_listen error for %s, %s", host, serv);
		exit (-1);
	}

        Listen(listenfd, LISTENQ);

        if (addrlenp)
                *addrlenp = res->ai_addrlen;    /* return size of protocol address */

        freeaddrinfo(ressave);

        return(listenfd);
}

